#!/usr/bin/python

"""
fcgi/web2ldap.py - stub script for running as FastCGI server

web2ldap -  web-based LDAP Client, see http://www.web2ldap.de
(c) by Michael Stroeder <michael@stroeder.com>

This software is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)
"""

import sys,os,time

exec_startdir = os.path.dirname(os.path.dirname(os.path.abspath(sys.argv[0])))
sys.path.insert(0,os.sep.join([exec_startdir,'etc','web2ldap']))
sys.path.insert(0,os.sep.join([exec_startdir,'pylib']))

if os.name == 'posix':
  # For finding web2ldapcnf.py in /etc/web2ldap on Unix systems
  sys.path.insert(0,'/etc/web2ldap')
else:
  # For finding web2ldapcnf.py in system dirs on Windows systems
  for i in ['SystemRoot','windir']:
    if os.environ.has_key(i):
      sys.path.insert(0,os.environ[i])

import web2ldapcnf
for i in web2ldapcnf.pylibdirs:
  sys.path.insert(0,i)

import w2lhandler,w2lcore

import fcgi,threading

class FastCGIThread(threading.Thread):
  """Thread class for FastCGIServer threads"""

  def __init__(self,req):
    self._req = req
    threading.Thread.__init__(self)
    self.started=time.time()
    self.setName(
      self.__class__.__name__+self.getName()[6:]
    )

  def __repr__(self):
    return '%s:%s - %s started %s' % (
      self._req.env.get('REMOTE_ADDR','unknown'),
      self._req.env.get('REMOTE_PORT','unknown'),
      self.getName(),
      time.strftime('%Y-%m-%dT%H:%M:%SZ',time.gmtime(self.started)),
    )

  def run(self):
    """create a new thread to handle requests"""
    try:
      w2lhandler.HandleHTTPRequest(
        self._req.inp,self._req.out,self._req.err,self._req.env
      )
    finally:
      self._req.Finish()
    return


# Create instance of Thread object for cleaning-up expired sessions
cleanUpThread = w2lcore.CleanUpThread(
  w2lcore.session,
  interval=5,
  session_expire=web2ldapcnf.session_relogin,
  session_remove=web2ldapcnf.session_remove
)
cleanUpThread.setDaemon(0)
# Start the clean-up thread
cleanUpThread.start()

while fcgi.isFCGI():
  req = fcgi.FCGI()
  t = FastCGIThread(req)
  t.start()

