"""Shelves derived from module shelve.

Mainly the differences are:
- set file mode (default 0600!)
- use bin flag when pickling
- use dumps and loads functions instead of StringIO objects
- check if filename points to a symbolic link

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)
"""

# Try using cPickle and cStringIO if available.

import os,shelve

try:
        from cPickle import loads, dumps
except ImportError:
        from pickle import loads, dumps


class Shelf(shelve.Shelf):
        """Base class for shelf implementations.

        This is initialized with a dictionary-like object.
        See the module's __doc__ string for an overview of the interface.
        """
        def __getitem__(self, key):
                return loads(self.dict[key])

        def __setitem__(self, key, value):
                self.dict[key] = dumps(value,1)
            

class DbfilenameShelf(Shelf):
        """Shelf implementation using the "anydbm" generic dbm interface.

        This is initialized with the filename for the dbm database.
        See the module's __doc__ string for an overview of the interface.
        """
        
        def __init__(self, filename, flag='c', mode=0600):
          import anydbm
          Shelf.__init__(self, anydbm.open(filename, flag, mode))
          if os.path.islink(filename):
            raise ValueError, "Security problem: Shelve database %s is a symbolic link!" % (
              filename
            )


def open(filename, flag='c', mode=0600):
        """Open a persistent dictionary for reading and writing.

        Argument is the filename for the dbm database.
        See the module's __doc__ string for an overview of the interface.
        """
        
        return DbfilenameShelf(filename, flag, mode)
