"""
w2ladd.py: add an entry

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import sys, ldap, ldif, \
       msbase,cgiforms,ldapbase,w2lcnf,w2lcore,web2ldapcnf,w2lgui,w2laddmodifyform
from msbase import listremove
try:
  from cStringIO import StringIO
except ImportError:
  from StringIO import StringIO

########################################################################
# Add new entry
########################################################################

def w2l_Add(sid,outf,command,form,ls):

  def AddListTable(form,ls,modlist):
    stringlist = []
    stringlist.append('<TABLE BORDER RULES=rows CELLSPACING=0 CELLPADDING=2%%>')
    for i in modlist:
      if i[0] in w2lcore.ldap_binaryattrkeys:
        tablestr = '%s bytes of binary data' % (
          ' + '.join(
            map(
              lambda x:'%d' % len(x),
              i[1]
            )
          )
        )
      else:
        tablestr = '<br />'.join(
          ldapbase.encode_unicode_list(
            ldapbase.unicode_list(i[1],ls.charset),
            form.accept_charset
          )
        )
      stringlist.append('<TR><TD>%s</TD><TD>%s</TD></TR>' % (
        w2lgui.AttrNameStr(i[0]),
	tablestr
      )
    )
    stringlist.append('</TABLE>')
    return '\n'.join(stringlist)

  if 'ldap_oc' in form.inputkeys:
    objectclasses = form.field['ldap_oc'][0].content
  else:
    raise w2lcore.ErrorExitClass(ls,'No object class(es) defined.')

  new_rdn_attr,default_objectclasses,dit_immutable = w2lcnf.GetParam(
    ls,'dit',{}
  ).get(
    ls.dn,('',[],0)
  )

  entry = msbase.CaseinsensitiveStringKeyDict()

  # Input from LDIF text area?
  if 'input_ldif' in form.inputkeys:
    f = StringIO(
      form.field['input_ldif'][0].content.encode(form.accept_charset)
    )
    try:
      ldif_data = ldif.ParseLDIF(f,maxentries=1)
    except ValueError:
      raise w2lcore.ErrorExitClass(ls,'Error parsing LDIF data.')
    else:
      if ldif_data:
        entry.update(
          ldif_data[0][1]
        )

  if entry.has_key('objectClass'):
    entry['objectClass'].extend(objectclasses)
  else:
    entry['objectClass'] = objectclasses

  if 'input_attr' in form.inputkeys:
    for i in range(len(form.field['input_attr'][0].content)):
      attr = form.field['input_attr'][0].content[i]
      if attr:
	data = form.field['input_data'][0].content[i]
	if data:
	  if entry.has_key(attr):
	    entry[attr].append(data)
	  else:
	    entry[attr] = [data]

  if not dit_immutable and \
     ('add_rdn' in form.inputkeys) and \
     form.field['add_rdn'][0].content:
    rdn = form.field['add_rdn'][0].content
    if not ldapbase.is_dn(rdn):
      w2laddmodifyform.w2l_AddModifyForm(
        sid,outf,command+'form',form,ls,
        Msg='<p>Wrong format of RDN string.</p>',
        rdn_default=rdn,
        entry=entry,
      )
      return
  else:
    if entry.has_key(new_rdn_attr):
      rdn='%s=%s' % (new_rdn_attr,entry[new_rdn_attr][0])
    else:
      w2laddmodifyform.w2l_AddModifyForm(
        sid,outf,command+'form',form,ls,
        Msg='<p>Attribute <strong>%s</strong> required for default RDN not in entry data.</p>' % (new_rdn_attr),
        rdn_default='',
        entry=entry,
      )
      return

  if not rdn:
    w2laddmodifyform.w2l_AddModifyForm(
      sid,outf,command+'form',form,ls,
      Msg='<p>Unable to determine new RDN.</p>',
      rdn_default='',
      entry=entry,
    )
    return

  # Modify-Liste zumsammenbauen
  modlist = ldapbase.add_modifylist(
    entry,
    binary_attrtypes=w2lcore.ldap_binaryattrkeys
  )

  if not modlist:
    raise w2lcore.ErrorExitClass(ls,'Cannot add entry without attributes.')

  if ls.dn:
    new_dn = ','.join([rdn,ls.dn])
  else:
    # Makes it possible to add entries for a namingContext
    new_dn = rdn

  # Add new entry
  try:
    ls.addEntry(new_dn,modlist)
  except ldap.NO_SUCH_OBJECT,e:
    parent_dn = ldapbase.ParentDN(new_dn)
    missing_parentdns = []
    while parent_dn.lower():
      try:
        parent_entry = ls.getAttributeTypes(
          parent_dn,w2lcnf.GetParam(ls,'search_attrsonly',1)
        )
      except ldap.NO_SUCH_OBJECT:
        missing_parentdns.append(parent_dn)
      parent_dn = ldapbase.ParentDN(parent_dn)
    if missing_parentdns:
      w2lgui.PrintHeader(outf,'Error',form.accept_charset)
      w2lgui.MainMenuTable(sid,outf,form,ls)
      outf.write("""<div id=MessageDiv>
        <h1>Error</h1>
        <p>No such object: The entry %s was not correctly added.</p>
        Missing parent entries:
        <table>""" % (
          w2lgui.DisplayDN(sid,form,ls,new_dn)
        )
      )
      for parent_dn in missing_parentdns:
        rdn,dn = ldapbase.SplitRDN(parent_dn)
        outf.write('<tr><td>%s</td><td>%s</td></tr>' % (
	    parent_dn,
	    w2lgui.CommandButton(
	      form,'addform','Create entry',sid,
	      dn=dn,
              hidden_fields = [('addform_rdn',rdn)],
              target='_inputform'
            )
          )
        )
      outf.write('</table></div>')
      w2lgui.PrintFooter(outf)
      return
    else:
      raise e
  except ldap.ALREADY_EXISTS:
    w2laddmodifyform.w2l_AddModifyForm(
      sid,outf,command+'form',form,ls,
      Msg='<p>Entry %s already exists. You probably did not define a non-existent RDN.</p>' % (
        w2lgui.DisplayDN(sid,form,ls,new_dn,commandbutton=0)
      ),
      rdn_default=rdn,
      entry=entry,
    )
  except ldap.OBJECT_CLASS_VIOLATION:
    w2laddmodifyform.w2l_AddModifyForm(
      sid,outf,command+'form',form,ls,
      Msg='<p>Object class violation: Maybe a required attribute is missing or some of the given attributes are not allowed.</p>',
      rdn_default=rdn,
      entry=entry,
    )
  else:
    w2lgui.PrintHeader(outf,'',form.accept_charset)
    w2lgui.MainMenuTable(sid,outf,form,ls)
    outf.write(
      '<div id=MessageDiv>\n<p>Added entry %s with your input data:</p>\n%s\n</div>\n' % (
        w2lgui.DisplayDN(sid,form,ls,new_dn,commandbutton=1),
        AddListTable(form,ls,modlist)
      )
    )
    w2lgui.PrintFooter(outf)
