"""
w2laddmodifyform.py: input form for adding and modifying an entry

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import sys, \
       cgiforms, ldap, ldapbase, w2lcore, w2lcnf, web2ldapcnf, msbase, w2lgui
from ldapbase import ParentDN
from msbase import union,listremove
from ldapoc import ldap_oc

heading_msg = {'modifyform':'Modify entry','addform':'Add new entry'}


def w2l_AddModifyForm(sid,outf,command,form,ls,Msg='',rdn_default='',entry={}):

  def ObjectClassForm(sid,outf,form,command,ls,rdn):
    """Form for choosing object class(es)"""
    if rdn:
      command_hidden_fields = [('addform_rdn',rdn)]
    else:
      command_hidden_fields = []
    Msg = {
      'addform':'Choose object class(es) for new entry.',
      'modifyform':'You may change the object class(es) for the entry.',
    }[command]
    outf.write('<p>%s</p>\n<table summary="Chossing object classes">\n<tr>\n' % (Msg))
    # Select list with all configured object classes
    outf.write('<td align="middle">%s</td>\n' % (
      w2lgui.CommandButton(
	form,command,'Input form',sid,
        hidden_fields=command_hidden_fields,
	extrastr='<br />%s\n' % (
          form.field['ldap_oc'][0].inputfield()
        )
      )    
    ))
    if command=='addform':
      outf.write('<td align="middle" valign="top">\n')
      default_oc_list = w2lcnf.GetParam(ls,'addform_oc_list',[])
      for desc,oc_list in default_oc_list:
        ldap_oc_field = '\n'.join(
          [
            '<input type="hidden" name="ldap_oc" value="%s">' % (oc)
            for oc in oc_list
          ]
        )
        outf.write(
          w2lgui.CommandButton(
	    form,command,desc,sid,
            hidden_fields=command_hidden_fields,
	    extrastr=ldap_oc_field
          )
        )
        outf.write('%s<br />\n' % (', '.join(oc_list)))
      outf.write('</td>\n')
    outf.write('</tr>\n</table>\n')

  def PrintInputTable(outf,form,attrs,tableheader='',entry={}):
    outf.write("""<fieldset title="%s">
      <table width="100%%">
      <tr><th colspan=2>%s</th></tr>""" % (
        tableheader,tableheader
      )
    )
    for attr in attrs:
      datalist = entry.get(attr.lower(),entry.get(attr,['']))
      for data in datalist:
	outf.write("""<tr>
            <td width="20%%">
              <label for="%s">%s</label>
              <input type=hidden name="input_attr" value="%s">
            </td>
            <td>
              <input name="input_data" id="%s" value="%s" size=50>
            </td>
          </tr>
          """ % (
            attr,
            w2lgui.AttrNameStr(attr),
            attr,
            attr,
            w2lcore.utf2display(form.accept_charset,data)
          )
        )
    outf.write('</table></fieldset>')
    return

  known_objectclasses = ldap_oc.keys()
  known_objectclasses.sort()

  if command=='addform':
    new_rdn_attr,objectclasses,dit_immutable = w2lcnf.GetParam(ls,'dit',{}).get(ls.dn,(None,[],0))
    form.field['ldap_oc'][0].setdefault(objectclasses)
    if 'addform_rdn' in form.inputkeys:
      add_rdn = form.field['addform_rdn'][0].content
    else:
      if rdn_default:
        add_rdn = rdn_default
      else:
        add_rdn = ''
    add_rdn_field = cgiforms.formInputClass('add_rdn',u'RDN of new entry',255,'.*',default=add_rdn,size=40)
  elif command=='modifyform':
    add_rdn = ''
    new_rdn_attr,objectclasses,dit_immutable = w2lcnf.GetParam(ls,'dit',{}).get(ParentDN(ls.dn),('',[],0))

  if not dit_immutable:
    # User is allowed to change object class
    if entry:
      # Get objectclass(es) from entry parameter
      objectclasses = entry.get('objectClass',entry.get('objectclass',[]))
    elif 'ldap_oc' in form.inputkeys:
      # Read objectclass(es) from input form
      objectclasses = form.field['ldap_oc'][0].content
    else:
      # User did not specify ldap_oc
      # => output a form for selecting object class(es)
      if command=='modifyform':
        # read object classes from existing entry
        existing_objectclasses = ls.getObjectClasses(ls.dn)
        # Set the selected defaults to existing object classes
	form.field['ldap_oc'][0].setdefault(
          ldapbase.unicode_list(existing_objectclasses,'ascii')
        )
	form.field['ldap_oc'][0].options.sort()

      w2lgui.PrintHeader(outf,'',form.accept_charset)
      w2lgui.MainMenuTable(sid,outf,form,ls)
      outf.write('<div id=InputDiv>\n<h1>%s</h1>' % (heading_msg[command]))
      ObjectClassForm(sid,outf,form,command,ls,add_rdn)
      outf.write('</div>\n')
      w2lgui.PrintFooter(outf)
      return


  if (command=='modifyform'):
    if entry:
      read_attrs = entry.keys()
    else:
      # Retrieve attribute types of the entry first
      # Get all attribute names of entries
      read_attrs = ls.getAttributeTypes(
        ls.dn,w2lcnf.GetParam(ls,'search_attrsonly',1)
      )
      # Falls nicht nur ein Attributinhalt gelesen werden soll,
      # so werden alle Binaerattribute, fuer die ein
      # MIME-Typ-Mapping angegeben wurde, rausgestrichen
      listremove(read_attrs,w2lcore.ldap_binaryattrkeys,1)
      # Read all non-binary attributes of entry
      entry = ls.readEntry(ls.dn,read_attrs)[0][1]

  ActionStr = '%s%s' % (command[0].upper(),command[1:-4])

  required_attrs = []
  allowed_attrs = []
  # Make a case-ignoring string-keyed copy of ldap_oc
  ldap_oc_ci = msbase.CaseinsensitiveStringKeyDict(ldap_oc)
  for oc in objectclasses:
    oc_def = ldap_oc_ci.get(oc,{})
    required_attrs = union(required_attrs,oc_def.get('requires',[]))
    allowed_attrs = union(allowed_attrs,oc_def.get('allows',[]))

  donottouch_attrs = ['objectclass']+w2lcnf.GetParam(ls,'hiddenattrs',[])
  listremove(required_attrs,donottouch_attrs,1)
  listremove(required_attrs,w2lcore.ldap_binaryattrkeys,1)
  listremove(allowed_attrs,donottouch_attrs,1)
  listremove(allowed_attrs,w2lcore.ldap_binaryattrkeys,1)
  listremove(allowed_attrs,required_attrs,1)

  w2lgui.PrintHeader(outf,'',form.accept_charset)
  w2lgui.MainMenuTable(sid,outf,form,ls)
  outf.write("""<div id=InputDiv>
    <h1>%s</h1>
    %s
    <form 
      action="%s/%s"
      method="POST"
      enctype="application/x-www-form-urlencoded"
      accept-charset="%s"
    >
    <input type="hidden" name="sid" value="%s">
    <input type=hidden name="dn" value="%s">
    <input type="submit" value="%s"><input type="reset" value="Reset">
    """ % (
      heading_msg[command],
      Msg,
      form.script_name,command[:-4],
      form.accept_charset,
      sid,
      ls.dn.encode(form.accept_charset),
      ActionStr
    )
  )

  if command=='addform':
    if add_rdn or (not new_rdn_attr or not dit_immutable):
      outf.write('<p>RDN: %s</p>' % add_rdn_field.inputfield())

  # Print chosen objectclass(es) as hidden input
  for oc in objectclasses:
    outf.write('<input type=hidden name="ldap_oc" value="%s">\n' % (oc))

  outf.write("""
<table
  id=InputTable
  summary="Input form for LDAP entry data"
  rules=rows>
<tr><td>%s</td><td>%s</td></tr>
</table>
"""% ( \
    w2lgui.AttrNameStr('objectclass'),
    '<br>'.join(objectclasses)
  ))

  PrintInputTable(outf,form,required_attrs,'Required attributes',entry)
  PrintInputTable(outf,form,allowed_attrs,'Allowed attributes',entry)
  if command=='modifyform':
    # Print attributes which does not appear in the schema but
    # do exist in the entry
    listremove(read_attrs,['objectclass','modifytimestamp','creatorsname','createtimestamp','modifiersname','userpassword'],1)
    listremove(read_attrs,required_attrs,1)
    listremove(read_attrs,allowed_attrs,1)
    if read_attrs:
      PrintInputTable(outf,form,read_attrs,'Existing attributes not matching schema',entry)

  outf.write("""<fieldset title="Raw LDIF data">
    <p>
      <em>Raw LDIF data</em>
    </p>
    <p>
      <textarea name="input_ldif" value="" wrap=off rows=15 cols=75></textarea><br />
    </p>
    <p>
      Notes:<br />
      <ul>
        <li>Lines containing "dn:" will be ignored</li>
        <li>Only the first entry (until first empty line) will be accepted</li>
        <li>NON-ASCII characters are not allowed!</li>
        <li>maximum length is set to %d bytes
      </ul>
    </p>
  </fieldset>
</form>
</div>
""" % (web2ldapcnf.ldif_maxbytes))

  w2lgui.PrintFooter(outf)

