"""
w2lsecinfo.py: Display (SSL) connection data

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import time,ldap,web2ldapcnf,cgissl,w2lcore,w2lgui,w2lcnf,msgzip

##############################################################################
# Connection info
##############################################################################

def w2l_ConnInfo(sid,outf,command,form,ls):

  context_menu_list = []
  for dn,txt in [
    ('','Root DSE'),('CN=MONITOR','CN=MONITOR'),('CN=CONFIG','CN=CONFIG')
  ]:
    try:
      if ls.existingEntry(dn,suppress_referrals=1):
        context_menu_list.append(
          w2lgui.CommandButton(
            form,'read',txt,
            sid,
            dn=dn
          )
        )
    except ldap.LDAPError:
      pass

  w2lgui.PrintHeader(outf,'Connection Info',form.accept_charset)
  w2lgui.MainMenuTable(sid,outf,form,ls)
  
  w2lgui.CommandTable(
    outf,context_menu_list,div_id='ContextMenuDiv',table_id='ContextMenuTable'
  )

  if ls.who:
    who_str = w2lgui.DisplayDN(sid,form,ls,ls.who,commandbutton=1)
  else:
    who_str = 'anonymous access'

  uptime = (time.time()-w2lcore.startUpTime)/60

  outf.write("""
    <div id=MessageDiv>
    <h1>Connection Info</h1>
    <h2>Gateway statistics</h2>
    <table>
    <tr>
      <td>Uptime:</td>
      <td>%s</td>
    </tr>
    <tr>
      <td>LDAP connections:</td>
      <td>%d</td>
    </tr>
    </table>
    <h2>LDAP</h2>
    <table>
    <tr>
      <td>LDAPv%d connection to:</td>
      <td>%s (%s)</td>
    </tr>
    <tr>
      <td>Bound as:</td>
      <td>%s</td>
    </tr>
    <tr>
      <td>Character set/encoding:</td>
      <td>%s</td>
    </tr>
    <tr>
      <td>Cached entries:</td>
      <td>%d</td>
    </tr>
    </table>
    """ % (
      '%02d:%02d' % (uptime/60,uptime%60),
      len(w2lcore.session.sessiondict)/2,
      ls.currentLDAPVersion,
      ls.host.encode('ascii'),ls.hostAddress.encode('ascii'),
      who_str,
      ls.charset.upper(),
      len(ls.read_cache),
    )
  )
  if msgzip.GzipFile and isinstance(outf,msgzip.GzipFile):
    gzip_level = web2ldapcnf.gzip_level
  else:
    gzip_level = 0
  outf.write("""
    <h2>HTTP</h2>
    <table>
      <tr><td>Your IP address:</td><td>%s</td></tr>
      <tr><td>Server signature:</td><td>%s</td></tr>
      <tr><td>Character set/encoding:</td><td>%s</td></tr>
      <tr><td>GZIP compression:</td><td>%d</td></tr>
    </table>
    <h2>SSL</h2>
    """ % (
      form.env.get('REMOTE_ADDR','???'),
      form.env.get('SERVER_SIGNATURE',''),
      form.accept_charset.upper(),
      gzip_level,
    )
  )
  sec_valid_dn = w2lcnf.GetParam(ls,'sec_valid_dn','')
  sec_valid_idn = w2lcnf.GetParam(ls,'sec_valid_idn','')
  cgissl.PrintSecInfo(web2ldapcnf.sec_sslacceptedciphers,sec_valid_dn,sec_valid_idn,outf)
  outf.write('</div>')
  w2lgui.PrintFooter(outf)
