"""
w2lhandler.py: base handler

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import sys,os,time,socket,msshelve,msgzip,\
       ldap,\
       cgiforms,ldapbase,ldapsession,ldif,web2ldapcnf, \
       w2lcore,w2lgui,w2lcnf,cgisession,httphelper

# Import the application modules
import w2lpasswd,w2lsearchform,w2llocate,w2lsearch,w2laddmodifyform,\
       w2ladd,w2lmodify,w2ldelete,w2lmodrdn,w2lpasswd,w2lread,\
       w2lconninfo,w2llogin,w2lconnect,w2lreferral,w2lmonitor

def HandleHTTPRequest(
  inf=sys.stdin,
  outfile=sys.stdout,
  errfile=sys.stderr,
  env=os.environ
):

  def InitializeForm(command,inf,env,accept_charset):
    """
    Return an instantiated cgiforms.formClass object
    depending on command
    """
    form = cgiforms.formClass(inf,env,accept_charset)
    form.add(cgiforms.formInputClass('sid',u'Session ID',16,u'[a-f0-9]+'))
    form.add(cgiforms.formInputClass('who',u'Bind DN',255,u'.*',size=40))
    form.add(cgiforms.formInputClass('cred',u'with Password',30,u'.*',size=15))
    form.add(cgiforms.formInputClass('host',u'Host:Port',255,ldapbase.host_pattern,size=30))
    form.add(w2lgui.DistinguishedNameInput('dn','DN'))
    if command=='searchform':
      form.add(cgiforms.formSelectClass('searchform_mode',u'Search form mode',[(u'base',u'Base'),(u'adv',u'Advanced'),(u'exp',u'Expert')],u'base'))
      form.add(cgiforms.formFieldClass('search_params',u'Anzahl Suchangaben',2,'[0-9]+'))
      form.add(w2lgui.DistinguishedNameInput('search_root','Search root'))
      form.add(cgiforms.formSelectClass('search_scope',u'Scope',[(u'base',u'Base'),(u'one',u'One level'),(u'sub',u'Sub tree')],u'sub'))
      form.add(cgiforms.formInputClass('search_filterstr',u'Search filter string',120,'.*'))
#      form.add(cgiforms.formSelectClass('search_resnumber',u'Number of results to display',[(u'0',u'unlimited'),(u'10',u'10'),(u'20',u'20'),(u'50',u'50'),(u'100',u'100'),(u'200',u'200')],u'10'))
    elif command=='search':
      form.add(cgiforms.formInputClass('search_filterstr',u'Search filter string',120,u'.*'))
      form.add(cgiforms.formInputClass('search_resminindex',u'Minimum index of search results',10,u'[0-9]+'))
      form.add(cgiforms.formInputClass('search_resnumber',u'Number of results to display',3,u'[0-9]+'))
      form.add(cgiforms.formSelectClass('search_output',u'Search output format',[u'table',u'ldif',u'ldif1',u'dsml',u'print']))
      form.add(cgiforms.formSelectClass('search_mode',u'Search Mode',[ur'(&%s)',ur'(|%s)']))
      form.add(cgiforms.formSelectClass('search_scope',u'Scope',[(u'base',u'Base'),(u'one',u'One level'),(u'sub',u'Sub tree')],u'sub'))
      form.add(cgiforms.formInputClass('search_attr',u'Search attribute',250,ur'[\w_;-]+',multiple=web2ldapcnf.max_searchparams))
      form.add(cgiforms.formSelectClass('search_option',u'Search option',w2lcore.search_options,multiple=web2ldapcnf.max_searchparams))
      form.add(cgiforms.formInputClass('search_string',u'Search string',60,u'.*',multiple=web2ldapcnf.max_searchparams))
    elif command in ['addform','modifyform']:
      if command=='addform':
        form.add(cgiforms.formInputClass('addform_rdn','RDN of new entry',255,ldapbase.rdn_pattern,size=40))
      form.add(w2lgui.ObjectClassSelect())
    elif command=='add':
      form.add(cgiforms.formInputClass('add_rdn',u'RDN of new entry',255,u'.*'))
      form.add(w2lgui.ObjectClassSelect())
      form.add(cgiforms.formInputClass('input_attr',u'Attribute Type',255,'[\w_;-]*',multiple=web2ldapcnf.input_maxattrs))
      form.add(cgiforms.formInputClass('input_data',u'Attribute Value',web2ldapcnf.input_maxfieldlen,'.*',multiple=web2ldapcnf.input_maxattrs))
      form.add(cgiforms.formTextareaClass('input_ldif',u'LDIF data',web2ldapcnf.ldif_maxbytes,ldif.ldif_pattern))
    elif command=='modify':
      form.add(w2lgui.ObjectClassSelect())
      form.add(cgiforms.formInputClass('input_attr',u'Attribute Type',255,u'[\w_;-]+',multiple=web2ldapcnf.input_maxattrs))
      form.add(cgiforms.formInputClass('input_data',u'Attribute Value',web2ldapcnf.input_maxfieldlen,'.*',multiple=web2ldapcnf.input_maxattrs))
      form.add(cgiforms.formTextareaClass('input_ldif',u'LDIF data',web2ldapcnf.ldif_maxbytes,ldif.ldif_pattern))
    elif command=='delete':
      form.add(cgiforms.formSelectClass('delete_confirm',u'Confirmation',['yes','no']))
      form.add(cgiforms.formCheckboxClass('delete_recursive',u'recursive delete','yes',checked=0))
      form.add(cgiforms.formCheckboxClass('delete_coe',u'continue on error','yes',checked=0))
    elif command=='modrdn':
      form.add(cgiforms.formInputClass('modrdn_newrdn',u'New RDN',255,ldapbase.rdn_pattern,size=50))
      form.add(cgiforms.formCheckboxClass('modrdn_delold',u'Delete old',value="yes",checked=1))
    elif command=='passwd':
      form.add(w2lgui.DistinguishedNameInput('passwd_who',u'Password DN'))
      form.add(cgiforms.formFieldClass('passwd_newpasswd',u'New password',100,'.*',multiple=2))
      form.add(cgiforms.formSelectClass('passwd_hash',u'Hash',w2lpasswd.available_hashtypes,w2lpasswd.available_hashtypes[-1]))
    elif command=='read' or command=='vcard':
      form.add(cgiforms.formInputClass('read_attr',u'Read attribute',255,u'[\w_;-]+'))
      form.add(cgiforms.formSelectClass('read_attrmode',u'Read attribute',[u'view',u'load']))
      form.add(cgiforms.formInputClass('read_attrindex',u'Read attribute',255,u'[0-9]+'))
      form.add(cgiforms.formInputClass('read_attrmimetype',u'MIME type',255,u'[\w.-]+/[\w.-]+'))
    elif command=='login':
      form.add(w2lgui.DistinguishedNameInput('login_who',u'Bind DN'))
    elif command=='locate':
      form.add(
        cgiforms.formInputClass(
          'locate_name',
          u'Location name',
          255,
          u'(%s|%s|%s)' % (
            w2lgui.mail_pattern[1:-1],
            ldapbase.host_pattern,
            ldapbase.dn_pattern
          ),
          size=25
        )
      )
    return form

  def CommandDispatcher(sid,outf,command,form,ls):
    """Execute function for command"""
    if command=='ldapurl':
      return
    if command=='searchform':
      w2lsearchform.w2l_SearchForm(sid,outf,command,form,ls)
    elif command=='locate':
      w2llocate.w2l_Locate(sid,outf,command,form,ls)
    elif command=='search':
      w2lsearch.w2l_Search(sid,outf,command,form,ls)
    elif command=='addform' or command=='modifyform':
      w2laddmodifyform.w2l_AddModifyForm(sid,outf,command,form,ls)
    elif command=='add':
      w2ladd.w2l_Add(sid,outf,command,form,ls)
    elif command=='modify':
      w2lmodify.w2l_Modify(sid,outf,command,form,ls)
    elif command=='delete':
      w2ldelete.w2l_Delete(sid,outf,command,form,ls)
    elif command=='modrdn':
      w2lmodrdn.w2l_ModRDN(sid,outf,command,form,ls)
    elif command=='passwd':
      w2lpasswd.w2l_Passwd(sid,outf,command,form,ls)
    elif command=='read' or command=='vcard':
      w2lread.w2l_Read(sid,outf,command,form,ls)
    elif command=='conninfo':
      w2lconninfo.w2l_ConnInfo(sid,outf,command,form,ls)
    elif command=='login':
      w2llogin.w2l_Login(sid,outf,'searchform',form,ls)
    else:
      raise w2lcore.ErrorExitClass(
        ls,'Invalid web2ldap command <strong>%s</strong>!' % (command)
      )
    return

  # Extract the command from PATH_INFO env var
  command = env.get('PATH_INFO','/')[1:]

  if command=='urlredirect':
    httphelper.URLRedirect(outfile,env.get('QUERY_STRING',''))
    return

  if ('gzip' in env.get('HTTP_ACCEPT_ENCODING','').lower().split(',')) and \
     (web2ldapcnf.gzip_level>0) and \
     command!='vcard':
    if msgzip.GzipFile:
      # Create new gzip-stream file object
      outf = msgzip.GzipFile(
        filename='web2ldap output',
        mode='wb',
        compresslevel=web2ldapcnf.gzip_level,
        fileobj=outfile
      )
    else:
      outf = outfile
  else:
    outf = outfile

  http_accept_charset = env.get('HTTP_ACCEPT_CHARSET')
  if web2ldapcnf.print_rawutf8==2:
    accept_charset = 'utf-8'
  elif web2ldapcnf.print_rawutf8==1 and http_accept_charset:
    if 'utf-8' in http_accept_charset.split(','):
      accept_charset = 'utf-8'
    else:
      accept_charset = 'iso-8859-1'
  else:
    accept_charset = 'iso-8859-1'

  sid = None;ls=None
  form = InitializeForm(command,inf,env,accept_charset)

  # Main block
  try:

    # Handle HTML form input if not started by LDAP URL
    if command!='ldapurl':
      form.getparams(ignoreemptyparams=0)

    if command=='':
      # New connect => create new session, remove old if necessary
      if 'sid' in form.inputkeys:
        sid = form.field['sid'][0].content.encode('ascii')
        if w2lcore.session.sessiondict.has_key(sid):
          # Delete old session
          w2lcore.session.deleteSession(sid)
      # Create new session
      ls = ldapsession.LDAPSession()
      w2lconnect.w2l_Connect(outf,command,form,ls)
      return

    elif command=='monitor':
      w2lmonitor.w2l_Monitor(outf,command,form)
      return

    elif command=='ldapurl':
      # Started by LDAP URL => create new session
      sid = w2lcore.session.newSession(env)
      ls = ldapsession.LDAPSession()
      ldap_url = env.get('QUERY_STRING','')
      if ldap_url:
        if ldapbase.is_ldap_url(ldap_url):
          host,dn,search_attr,search_scope,search_filterstr,ext = ldapbase.parse_ldap_url(ldap_url)
          try:
            dn = unicode(dn,'utf-8')
          except UnicodeError:
            dn = unicode(dn,'iso-8859-1')
          if not host:
            raise w2lcore.ErrorExitClass(ls,'LDAP URL without host name currently not supported.')
          ls.open(host)
          ls.set_option('deref',0);ls.set_option('referrals',0)
          # Search and set host specific timeout
          ls.timeout = w2lcnf.GetParam(ls,'timeout',-1)
          ls.getRootDSE()
          ls.bind('','',ldap.AUTH_SIMPLE)
          ls.setDN(dn)
          # Search and set backend specific timeout
          ls.timeout = w2lcnf.GetParam(ls,'timeout',-1)
          # Search and set backend specific cache options
          ls.setCache(
            w2lcnf.GetParam(ls,'cache_timeout',0),
            w2lcnf.GetParam(ls,'cache_maxmem',0),
          )
          if search_scope==ldap.SCOPE_BASE:
            w2lread.w2l_Read(sid,outf,'read',form,ls,search_attr)
          else:
            if not search_filterstr: search_filterstr='(objectClass=*)'
            w2lsearch.w2l_Search(sid,outf,'search',form,ls,'table',search_scope,search_filterstr)
        elif ldap_url.startswith('ldaps:'):
          raise w2lcore.ErrorExitClass(ls,'LDAP over SSL not supported.')
        else:
          raise w2lcore.ErrorExitClass(ls,'LDAP URL has invalid format.')
      else:
        raise w2lcore.ErrorExitClass(ls,'No LDAP URL specified.')

    else:
      if 'sid' in form.inputkeys:
        sid = form.field['sid'][0].content.encode('ascii')
        try:
          ls = w2lcore.session.retrieveSession(sid,env)
        except cgisession.SessionExpired,e:
          # Handle expired sessions
          if hasattr(e.session_data,'host'):
            # Allow to relogin
            sid = w2lcore.session.newSession(env)
            ls = e.session_data ; ls.cred = ''
            w2llogin.w2l_Login(
              sid,outf,command,form,ls,
              login_msg=str(e),who=ls.who,relogin=1,nomenu=1
            )
          else:
            httphelper.URLRedirect(outf,form.script_name,2,'Session expired.')
          return
        except cgisession.KeyError:
          httphelper.URLRedirect(outf,form.script_name,2,'Session ID not found.')
          return
        except cgisession.SessionHijacked,e:
          httphelper.URLRedirect(
            outf,form.script_name,2,'Session hijacking detected. Access denied!'
          )
          return
      else:
        # Create new session
        sid = w2lcore.session.newSession(env)
        ls = ldapsession.LDAPSession()

      # Handle host parameter
      if 'host' in form.inputkeys:
        host = form.field['host'][0].content
        if not hasattr(ls,'host') or host!=ls.host:
          ls = ldapsession.LDAPSession()
          ls.open(form.field['host'][0].content)
          ls.set_option('deref',0);ls.set_option('referrals',0)
          ls.timeout = w2lcnf.GetParam(ls,'timeout',-1)
          ls.getRootDSE()
          if ('dn' in form.inputkeys):
            ls.setDN(form.field['dn'][0].content)
            # Search and set backend specific timeout
            ls.timeout = w2lcnf.GetParam(ls,'timeout',-1)
            # Search and set backend specific cache options
            ls.setCache(
              w2lcnf.GetParam(ls,'cache_timeout',0),
              w2lcnf.GetParam(ls,'cache_maxmem',0),
            )
      # Handle login parameter
      if ('who' in form.inputkeys) and ('cred' in form.inputkeys):
        who,cred = form.field['who'][0].content,form.field['cred'][0].content
        ls.bind(
          who,cred,
          filtertemplate=w2lcnf.GetParam(ls,'binddnsearch','(uid=%s)'),
          attrnamesonly=w2lcnf.GetParam(ls,'search_attrsonly',1),
        )
      else:
        who = ''
      if ('dn' in form.inputkeys):
        ls.setDN(form.field['dn'][0].content)

    # Flush the session into the database and close database
    w2lcore.session.storeSession(sid,ls)

    # Execute the command module
    CommandDispatcher(sid,outf,command,form,ls)

  except cgiforms.formException,e:
    w2lgui.ExceptionMsg(None,outf,form,None,'FORM handling error',str(e))
    return

  except ldap.PARTIAL_RESULTS,e:
    w2lreferral.w2l_ChaseReferral(sid,outf,command,form,ls,e)
  except ldap.INSUFFICIENT_ACCESS,e:
    w2llogin.w2l_Login(
      sid,outf,command,form,ls,who='',
      login_msg='Insufficient access. Retry with new login.',relogin=1
    )
  except ldap.INAPPROPRIATE_AUTH,e:
    w2llogin.w2l_Login(
      sid,outf,command,form,ls,who='',
      login_msg='Inappropriate authentication scheme.',relogin=1
    )
  except ldap.INVALID_CREDENTIALS,e:
    w2llogin.w2l_Login(
      sid,outf,command,form,ls,
      login_msg='Invalid credentials. Wrong password?',
      who=who,relogin=1
    )
  except ldap.SERVER_DOWN,e:
    if hasattr(ls,'who'):
      # Try to re-establish an old connection
      host=ls.host;dn=ls.dn;
      try:
        who=ls.who
      except AttributeError:
        who=''
      ls.unbind()
      ls = ldapsession.LDAPSession()
      try:
        # Try to reconnect
        ls.open(host)
      except ldap.LDAPError:
        # Reconnecting impossible => new login
        httphelper.URLRedirect(
          outf,form.script_name,2,
          'Reconnecting to host %s impossible.' % (host)
        )
      else:
        # Reconnecting was possible => new connect
        ls.set_option('deref',0);ls.set_option('referrals',0)
        ls.timeout = w2lcnf.GetParam(ls,'timeout',-1)
        ls.getRootDSE()
        ls.setDN(dn)
        w2llogin.w2l_Login(
          sid,outf,command,form,ls,
          login_msg='Connection to LDAP host %s invalid. Retry with new login.' % (ls.host.encode('ascii')),
          who=who,relogin=1,nomenu=1
        )
    else:
      # Reconnecting impossible => new login
      httphelper.URLRedirect(
        outf,form.script_name,2,
        'Connecting to host %s impossible.' % (host)
      )
  except ldapbase.USERNAME_NOT_FOUND,e:
    w2llogin.w2l_Login(sid,outf,command,form,ls,login_msg=str(e),who=who,relogin=1)
  except ldapbase.USERNAME_NOT_UNIQUE,e:
    w2llogin.w2l_Login(sid,outf,command,form,ls,login_msg=str(e),who=who,relogin=1)

  except ldap.LDAPError,e:
#  except None:
    if not vars().has_key('ls'):
      ls = ldapsession.LDAPSession()
    try:
      if type(e.args[0])==type({}):
        desc = e.args[0]['desc']
        if desc.lower()=='referral':
          w2lreferral.w2l_ChaseReferral(sid,outf,command,form,ls,e)
          return
        else:
          ErrMsg = '%s<br />%s' % (desc,e.args[0].get('info',''))
      elif type(e.args[0])==type(1):
        ErrMsg = 'Error Code: %d<br />Error description: %s' %(e.args[0],e.args[1])
      else:
        ErrMsg = str(e.args)
    except TypeError:
      ErrMsg = str(e.args)
    except IndexError:
      ErrMsg = str(e.args)
    w2lgui.ExceptionMsg(
      sid,outf,form,ls,'LDAP exception',ErrMsg
    )

  except socket.error,e:
    w2lgui.ExceptionMsg(sid,outf,form,None,'Socket Error',str(e))

  except IOError,e:
    w2lgui.ExceptionMsg(sid,outf,form,ls,'I/O Error',str(e))

  except w2lcore.ErrorExitClass,e:
    w2lgui.ExceptionMsg(sid,outf,form,ls,'Error',e.Msg)

  except:
    w2lcore.log_exception(errfile,sid,command,form,ls,env)

  w2lcore.session.storeSession(sid,ls)

  # Close gzip file-object if necessary
  if hasattr(outf,'fileobj'):
    outf.close()
    outf = None

  return
