"""
w2lmodify.py: modify an entry

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import sys,cgiforms,ldap,ldif,ldapbase,msbase,\
       w2lcore,web2ldapcnf,w2lcnf,w2lgui,w2laddmodifyform
try:
  from cStringIO import StringIO
except ImportError:
  from StringIO import StringIO

def GetEntryfromInputForm(ls,form):
  new_entry = msbase.CaseinsensitiveStringKeyDict({})
  # Input from LDIF text area?
  if 'input_ldif' in form.inputkeys:
    f = StringIO(
      form.field['input_ldif'][0].content.encode(form.accept_charset)
    )
    try:
      ldif_data = ldif.ParseLDIF(f,maxentries=1)
    except ValueError,e:
      raise w2lcore.ErrorExitClass(ls,'Error parsing LDIF data: %s' % str(e))
    else:
      if ldif_data:
        new_entry.update(
          ldif_data[0][1]
        )
  # Input from normal input fields
  if 'input_attr' in form.inputkeys:
    for i in xrange(len(form.field['input_attr'][0].content)):
      attr = form.field['input_attr'][0].content[i]
      if attr:
	data = form.field['input_data'][0].content[i]
	if new_entry.has_key(attr):
	  if data:
	    if new_entry[attr]==['']:
  	      new_entry[attr] = [data]
	    else:
	      new_entry[attr].append(data)
	else:
	  new_entry[attr] = [data]
  return new_entry  

##############################################################################
# Modify existing entry
##############################################################################

def w2l_Modify(sid,outf,command,form,ls):

  def ModifyListTable(form,ls,modlist):
    stringlist = []
    stringlist.append('<table border rules=rows cellspacing=0 cellpadding=2%%>')
    for i in modlist:
      if i[0]==ldap.MOD_ADD:
        if i[1] in w2lcore.ldap_binaryattrkeys:
	  tablestr = '%d Bytes of binary data' % (len(i[2]))
	else:
	  tablestr = unicode(i[2],ls.charset).encode(form.accept_charset)
      else:
        if i[1] in w2lcore.ldap_binaryattrkeys:
          tablestr = '%s bytes of binary data' % (
            ' + '.join(
              map(
                lambda x:'%d' % len(x),
                i[1]
              )
            )
          )
	else:
  	  tablestr = '<BR>'.join(
            ldapbase.encode_unicode_list(
              ldapbase.unicode_list(i[2],ls.charset),
              form.accept_charset
            )
          )
      stringlist.append('<tr><td>%s</td><td>%s</td><td>%s</td></tr>' % (
        ldapbase.MOD_STR[i[0]],w2lgui.AttrNameStr(i[1]),tablestr)
      )
    stringlist.append('</table>')
    return '\n'.join(stringlist)

  # Retrieve all attribute types
  existing_attrs = ls.getAttributeTypes(
    ls.dn,w2lcnf.GetParam(ls,'search_attrsonly',1)
  )
  # Remove the hidden attribute types
  msbase.listremove(
    existing_attrs,
    w2lcnf.GetParam(ls,'hiddenattrs',[]),
    ignorecase=1
  )
  # Remove the binary attribute types
  msbase.listremove(
    existing_attrs,
    w2lcore.ldap_binaryattrkeys,
    ignorecase=1
  )
  # Read modifiable attributes of entry
  result_dnlist = ls.readEntry(
    ls.dn,
    existing_attrs,
  )
  if not result_dnlist:
    raise w2lcore.ErrorExitClass(ls,"""
      No search result when reading entry <strong>%s</strong>.""" % (
        w2lgui.DisplayDN(sid,form,ls,ls.dn)
      )
    )
  old_entry = result_dnlist[0][1]

  new_entry = GetEntryfromInputForm(ls,form)

  old_objectclasses = old_entry.get('objectclass',old_entry.get('objectClass',[]))
  old_objectclasses.sort()

  modlist = []
  if 'ldap_oc' in form.inputkeys:
   # Object class(es) were defined in users input
    new_objectclasses = form.field['ldap_oc'][0].content
    new_objectclasses.sort()
    if map(lambda x: x.lower(),old_objectclasses)!=map(lambda x: x.lower(),new_objectclasses):
      modlist.append(
        (
          ldap.MOD_REPLACE,
          'objectClass',
          ldapbase.encode_unicode_list(new_objectclasses)
        )
      )

  modlist.extend(
    ldapbase.modify_modifylist(
      old_entry,
      new_entry,
      binary_attrtypes=w2lcore.ldap_binaryattrkeys
    )
  )

  if modlist:
    # Send modify-list to host
    try:
      ls.modifyEntry(ls.dn,modlist)
    except ldap.OBJECT_CLASS_VIOLATION:
      new_entry['objectClass'] = new_objectclasses
      w2laddmodifyform.w2l_AddModifyForm(
        sid,outf,command+'form',form,ls,
        Msg='<p>Object class violation: Maybe a required attribute is missing or some of the given attributes are not allowed.</p>',
        entry=new_entry,
      )
      return
    else:
      # delete all cache entries referencing to old DN
      UserMsg = 'Modified entry %s with your input data:</p>\n%s' % (
        w2lgui.DisplayDN(sid,form,ls,ls.dn),
        ModifyListTable(form,ls,modlist)
      )
  else:
    UserMsg = 'No attributes modified of entry %s.' % (w2lgui.DisplayDN(sid,form,ls,ls.dn))

  # Output comes here
  w2lgui.PrintHeader(outf,'',form.accept_charset)
  w2lgui.MainMenuTable(sid,outf,form,ls)
  w2lgui.ContextMenuSingleEntry(sid,outf,form,ls)
  outf.write('<div id=MessageDiv><p>%s</p></div>' % UserMsg)
  w2lgui.PrintFooter(outf)

