"""
w2lpasswd.py: change password associated with entry

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import sys,web2ldapcnf,w2lcnf,cgiforms,ldap,ldapbase,w2lcore,w2lgui,w2llogin

# Alphabet for encrypted passwords (see module crypt)
crypt_alphabet = './0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
crypt_alphabet_len = len(crypt_alphabet)

# Try to determine the avaiable hash types
# Check if all required modules are in place to
# determine the available password hash methods
available_hashtypes = []
try:
  import random
  import base64
  import sha
  available_hashtypes.append(('sha','SHA-1'))
  available_hashtypes.append(('ssha','salted SHA-1'))
  import md5
  available_hashtypes.append(('md5','MD5'))
  available_hashtypes.append(('smd5','salted MD5'))
  import crypt
  available_hashtypes.append(('crypt','Crypt'))
except ImportError:
  pass
available_hashtypes.append(('','None'))

def w2l_Passwd(sid,outf,command,form,ls):

  if 'passwd_who' in form.inputkeys:
    passwd_who = form.field['passwd_who'][0].content
  else:
    if ls.who:
      passwd_who = ls.who
    else:      
      passwd_who = ls.dn

  if passwd_who and \
     ('passwd_newpasswd' in form.inputkeys):

    if len(form.field['passwd_newpasswd'][0].content)!=2:
      raise w2lcore.ErrorExitClass(ls,'Repeat password!')

    if form.field['passwd_newpasswd'][0].content[0]!=form.field['passwd_newpasswd'][0].content[1]:
      raise w2lcore.ErrorExitClass(ls,'New passwords do not match!')

    if not form.field['passwd_newpasswd'][0].content[0]:
      raise w2lcore.ErrorExitClass(ls,'Empty passwords not allowed!')

    passwdinput = form.field['passwd_newpasswd'][0].content[0]

    if 'passwd_hash' in form.inputkeys:
      passwd_hash = form.field['passwd_hash'][0].content.lower()
      if passwd_hash in ['crypt','smd5','ssha']:
        crypt_salt = ''
        for i in range(2):
  	  crypt_salt = crypt_salt+crypt_alphabet[random.randint(0,crypt_alphabet_len-1)]
      else:
	crypt_salt = ''
      if passwd_hash=='crypt':
	new_passwd = '{crypt}%s' % crypt.crypt(form.field['passwd_newpasswd'][0].content[0],crypt_salt)
      elif passwd_hash in ['md5','smd5']:
	new_passwd = ('{%s}%s' % (passwd_hash,base64.encodestring(md5.new(passwdinput+crypt_salt).digest()+crypt_salt))).strip()
      elif passwd_hash in ['sha','ssha']:
	new_passwd = ('{%s}%s' % (passwd_hash,base64.encodestring(sha.new(passwdinput+crypt_salt).digest()+crypt_salt))).strip()
      else:
	new_passwd = passwdinput
    else:
      raise w2lcore.ErrorExitClass(ls,'Password encryption method required!')

    modlist = [(ldap.MOD_REPLACE,'userPassword',new_passwd.encode('utf-8'))]

    # Passwort aendern
    ls.modifyEntry(passwd_who,modlist)
    if ldapbase.normalize_dn(passwd_who.lower())==ldapbase.normalize_dn(ls.who.lower()):
      ls.bind('','')
      w2llogin.w2l_Login(
        sid,outf,'searchform',form,ls,
        login_msg='Password changed of entry %s. A new login is required!' % (w2lgui.DisplayDN(sid,form,ls,passwd_who)),
        who = passwd_who,relogin=0
      )
    else:
      w2lgui.PrintHeader(outf,'Changed password',form.accept_charset)
      w2lgui.MainMenuTable(sid,outf,form,ls)
      outf.write('<div id=messagediv>\n<p>Changed password of entry %s.</p>\n</div>' % (w2lgui.DisplayDN(sid,form,ls,passwd_who)))
      w2lgui.PrintFooter(outf)

  else:
    if ls.who:
      who = ls.who
    else:
      who = ls.dn
    config_hashtypes = w2lcnf.GetParam(ls,'passwd_hashtypes',[])
    if config_hashtypes:
      # The set of hash types are restricted by local configuration
      default_hashtypes = [
        hash_type
        for hash_type in available_hashtypes
        if hash_type[0] in config_hashtypes
      ]
      form.field['passwd_hash'][0].options = default_hashtypes

    # Leeres Passworteingabeformular ausgeben
    w2lgui.PrintHeader(outf,'Change password',form.accept_charset)
    w2lgui.MainMenuTable(sid,outf,form,ls)
    outf.write("""<div id=InputDiv>
<h1>Change password</h1>
<form
  action="%s/passwd"
  method="POST"
  enctype="application/x-www-form-urlencoded"
  accept-charset="%s"
>
  <input type="hidden" name="sid" value="%s">
  <input type="hidden" name="who" value="%s">
  <input type="hidden" name="passwd_who" value="%s">
  <table summary="Password input form">
  <tr>
    <td>Change password for</td>
    <td>%s on %s</td>
  </tr>
  <tr>
    <td>Password for <strong>%s</strong></td>
    <td><input type="password" name="cred" maxlength="30" size="15" value=""></td>
  </tr>
  <tr>
    <td>New password</td>
    <td><input type="password" name="passwd_newpasswd" maxlength="30" size="30" value=""></td>
  </tr>
  <tr>
    <td>repeat new password</td>
    <td><input type="password" name="passwd_newpasswd" maxlength="30" size="30" value=""></td>
  </tr>
  <tr>
    <td>Password encryption method</td>
    <td>%s</td>
  </tr>
</table>
<input type="submit" value="Change Password">
<input type="reset" value="Reset">
</form>
</div>
""" % (
    form.script_name,
    form.accept_charset,
    sid,
    w2lcore.utf2display(form.accept_charset,who),
    w2lcore.utf2display(form.accept_charset,passwd_who),
    w2lgui.DisplayDN(sid,form,ls,passwd_who),
    ls.host.encode('ascii'),
    w2lgui.DisplayDN(sid,form,ls,who),
    form.field['passwd_hash'][0].inputfield()
  )
)
    w2lgui.PrintFooter(outf)


