"""
w2lread.py: Read existing entry and output as HTML or vCard

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import sys,urllib,ldap, \
       msbase, cgiforms, msshelve, ldapbase, httphelper, \
       w2lcore, w2lcnf, web2ldapcnf, w2lgui,w2lread
from msbase import listremove,intersection,union

try:
  from ldapoidreg import oid
except ImportError:
  oid = {}

def w2l_Read(
  sid,
  outf,
  command,
  form,
  ls,
  wanted_attrs=None,
  read_attrmode = 'view',
  read_attrmimetype = None,
):

  def DisplayEntry(sid,form,ls,entry,oid={},commandbutton=0):
    display_entry = msbase.CaseinsensitiveStringKeyDict(default='&nbsp;')
    # Convert all values in attributes of LDAP entry
    # to pretty-printable HTML
    for attrtype,values in entry.items():
      display_entry[attrtype] = [
        w2lgui.DataStr(
          sid,form,ls,
          attrtype,values[i],
          valueindex=i,
          commandbutton=commandbutton,
          oid_reg=oid
        )
        for i in xrange(len(values))
      ]
    return display_entry

  def HTMLTemplateEntry(entry):
    template_entry = msbase.CaseinsensitiveStringKeyDict(default='&nbsp;')
    for attrtype,values in entry.items():
      template_entry[attrtype] = ', '.join(values)
    return template_entry

  def getOperationAttrsTemplate(ls,accept_language):
    template_filename = w2lgui.GetVariantFilename(
      w2lcnf.GetParam(ls,'read_operationalattrstemplate'),
      accept_language
    )
    if template_filename:
      return open(template_filename,'r').read()
    else:
      return """<table width="100%%">
  <tr>
    <td width="50%%">
      Entry created %(createTimeStamp)s by:
      %(creatorsName)s
    </td>
    <td width="50%%">
      Last modified %(modifyTimeStamp)s by:
      %(modifiersName)s
    </td>
  </tr>
</table>
"""

  def PrintAttrRows(sid,outf,form,ls,entry,attrs,read_attrs,Comment=''):
    if not attrs:
      return
    if Comment:
      outf.write('<tr><th colspan=2 align=left><strong>%s</strong></th></tr>\n' % (Comment))
    show_attrs = filter(lambda i,l=read_attrs:i in l,attrs)
    for attr in show_attrs:
      datalist = entry[attr]
      outf.write('<tr><td width="30%%" rowspan=%d>%s</td>' % (len(datalist),w2lgui.AttrNameStr(attr)))
      # Ausgabe des Attribut-Inhalts
      outf.write('<td align=left>%s</td></tr>\n' % datalist[0])
      for data in datalist[1:]:
	outf.write('<tr><td align=left>%s</td></tr>\n' % data)


  # Read the whole entry
  if ls.dn=='':
    # Load OID descriptions for RootDSE
    search_result = ls.readEntry(ls.dn,ldapbase.ROOTDSE_ATTRS)
  else:
    oid = {}
    search_result = ls.readEntry(ls.dn,None)

  if not search_result:
    raise w2lcore.ErrorExitClass(
      ls,
      'Empty search result.'
    )
  # Save session into database
  w2lcore.session.storeSession(sid,ls)

  # Determine all attribute types
  all_attrs = search_result[0][1].keys()
  # Attribute herausloeschen, welche unsichtbar bleiben sollen
  listremove(all_attrs,w2lcnf.GetParam(ls,'hiddenattrs',[]),ignorecase=1)

  # Wurde eine Attributwunschliste angegeben?
  if 'read_attr' in form.inputkeys:
    wanted_attrs = [form.field['read_attr'][0].content]
  else:
    if wanted_attrs is None:
      wanted_attrs = all_attrs

  entry = msbase.CaseinsensitiveStringKeyDict(search_result[0][1],None)

  if (not wanted_attrs) or len(wanted_attrs)>1 or \
     (not wanted_attrs[0].lower() in w2lcore.ldap_binaryattrkeys):
    operational_attrs_template = getOperationAttrsTemplate(ls,form.accept_language)
    operational_attrs = msbase.GrabKeys(operational_attrs_template)()
    operational_attrs.append('objectClass')
    operational_search_result = ls.readEntry(ls.dn,operational_attrs)
    operational_entry = msbase.CaseinsensitiveStringKeyDict(operational_search_result[0][1],[])
    entry['objectClass'] = operational_entry.get('objectClass',[])

  if command=='read':

    if (len(wanted_attrs)==1) and \
       (wanted_attrs[0].lower() in w2lcore.ldap_binaryattrkeys):

      attr_type = wanted_attrs[0]

      # Send a single binary attribute with appropriate MIME-type
      import w2lviewer
      if 'read_attrindex' in form.inputkeys:
        read_attrindex = int(form.field['read_attrindex'][0].content)
	if read_attrindex >= len(entry[attr_type]):
          raise w2lcore.ErrorExitClass(
            ls,
            'Illegal index for multi-valued binary attribute.'
          )
      else:
        read_attrindex = None
      if 'read_attrmode' in form.inputkeys:
        # Determine if user wants to view or download the binary attribute value
        read_attrmode = form.field['read_attrmode'][0].content
      if (read_attrmode=='view') and \
         (w2lviewer.viewer_func.has_key(attr_type)):
        # Nice displaying of binary attribute with viewer class
	w2lgui.PrintHeader(outf,ls.dn.encode(form.accept_charset),form.accept_charset)
	w2lgui.MainMenuTable(sid,outf,form,ls)
        outf.write('<div id=MessageDiv>\n')
        w2lviewer.viewer_func[attr_type](
	  sid,outf,command,form,ls,attr_type,entry,read_attrindex
	)
	outf.write('</div>\n')
	w2lgui.PrintFooter(outf)
      else:
        # Raw download with (hopefully) appropriate MIME-type
        if 'read_attrmimetype' in form.inputkeys:
          read_attrmimetype = form.field['read_attrmimetype'][0].content
        w2lviewer.DisplayBinaryAttribute(
          sid,outf,command,form,ls,attr_type,entry,read_attrindex,read_attrmimetype
	)

    else:

      # Display the whole entry
      w2lgui.PrintHeader(outf,w2lcore.utf2display(form.accept_charset,ls.dn),form.accept_charset)
      w2lgui.MainMenuTable(sid,outf,form,ls)
      w2lgui.ContextMenuSingleEntry(sid,outf,form,ls)

      objectclasses = entry.get('objectclass',entry.get('objectClass',[]))

      read_template_dict = msbase.CaseinsensitiveStringKeyDict(w2lcnf.GetParam(ls,'read_template',{}))

      template_oc = intersection(
        map(lambda x:x.lower(),objectclasses),
        map(lambda x:x.lower(),read_template_dict.keys()),
        ignorecase=1
      )
      # List of already displayed attributes
      displayed_attrs=[]
      display_entry = DisplayEntry(
        sid,form,ls,entry,w2lread.oid,commandbutton=1
      )

      outf.write('<div id=MessageDiv>\n')

      if template_oc:
        # Templates defined => display the entry with the help of the template
        used_templates = []
	for oc in template_oc:
	  read_template_filename = w2lgui.GetVariantFilename(read_template_dict[oc],form.accept_language)
          if not read_template_filename in used_templates:
            used_templates.append(read_template_filename)
            if read_template_filename:
	      try:
  	        template_str = open(read_template_filename,'r').read()
	      except IOError:
	        outf.write('<p>I/O error during reading template file for object class <strong>%s</strong>.</p>' % oc)
              else:
  	        outf.write(template_str % HTMLTemplateEntry(display_entry))
                displayed_attrs = msbase.union(displayed_attrs,msbase.GrabKeys(template_str)())
	    else:
              outf.write('<p>Template file for object class <strong>%s</strong> not found.</p>' % oc)

      # Remove attributes already displayed with HTML templates
      wanted_attrs = msbase.listremove(wanted_attrs,displayed_attrs,ignorecase=1)
      wanted_attrs = msbase.listremove(wanted_attrs,operational_attrs,ignorecase=1)

      if wanted_attrs:
	# display the attributes of the entry in a raw way
	from ldapoc import ldap_oc
	required_attrs = []
	allowed_attrs = []
	for oc in objectclasses:
	  oc_def = ldap_oc.get(oc,{})
	  required_attrs = union(required_attrs,oc_def.get('requires',[]))
	  allowed_attrs = union(allowed_attrs,oc_def.get('allows',[]))
	required_attrs = intersection(map(lambda x:x.lower(),required_attrs),wanted_attrs,ignorecase=1)
	allowed_attrs = intersection(map(lambda x:x.lower(),allowed_attrs),wanted_attrs,ignorecase=1)
	nomatching_attrs = wanted_attrs[:]
	listremove(nomatching_attrs,required_attrs,ignorecase=1)
	listremove(nomatching_attrs,allowed_attrs,ignorecase=1)
	listremove(required_attrs,['objectclass'],ignorecase=1)
	listremove(allowed_attrs,['objectclass'],ignorecase=1)
	listremove(nomatching_attrs,['objectclass'],ignorecase=1)

	# Eigentliche Ausgabe des gelesenen Eintrags
	outf.write('<table id=readtable summary="Content of LDAP entry" width="100%%" border rules=rows noshade cellpadding="2%%" cellspacing=0>')

        if operational_entry.has_key('objectClass'):
          PrintAttrRows(sid,outf,form,ls,operational_entry,['objectClass'],['objectClass'],'Object classes')
	PrintAttrRows(sid,outf,form,ls,display_entry,required_attrs,all_attrs,'Required Attributes')
	PrintAttrRows(sid,outf,form,ls,display_entry,allowed_attrs,all_attrs,'Allowed Attributes')
	PrintAttrRows(sid,outf,form,ls,display_entry,nomatching_attrs,all_attrs,'Attributes not matching schema')

	outf.write('</table>')

      outf.write(operational_attrs_template % (
        HTMLTemplateEntry(
            DisplayEntry(sid,form,ls,operational_entry,commandbutton=1)
          )
        )
      )
      outf.write('</div>\n')

      w2lgui.PrintFooter(outf)

  elif command=='vcard':

    objectclasses = entry.get('objectclass',entry.get('objectClass',[]))

    vcard_template_dict = msbase.CaseinsensitiveStringKeyDict(
      w2lcnf.GetParam(ls,'vcard_template',{})
    )
    template_oc = intersection(
      map(lambda x:x.lower(),objectclasses),
      map(lambda x:x.lower(),vcard_template_dict.keys()),
      ignorecase=1
    )

    if template_oc:

      # Templates defined => display the entry with the help of a template
      display_entry = msbase.CaseinsensitiveStringKeyDict(default='')
      display_entry['dn'] = ls.dn.encode('iso-8859-1')
      attr_list = filter(
        lambda i,l=w2lcore.ldap_binaryattrkeys:not i in l,
        entry.keys()
      )
      for attr in attr_list:
        itemlist = []
        for item in ldapbase.unicode_list(entry[attr]):
          itemlist.append('\, '.join(item.encode('iso-8859-1').split('$')))
	display_entry[attr] = ';'.join(itemlist)

      oc = template_oc[0]
      vcard_template_filename = w2lgui.GetVariantFilename(vcard_template_dict[oc],form.accept_language)
      if vcard_template_filename:
	try:
  	  template_str = open(vcard_template_filename,'r').read()
	except IOError:
	  raise w2lcore.ErrorExitClass(
            ls,
            """I/O error during reading template file for 
            object class <strong>%s</strong>.""" % (oc)
          )
        else:
          httphelper.SendHeader(
            outf,'text/x-vcard',
            charset='ISO-8559-1',
            expires_offset=web2ldapcnf.misc.sec_expire,
          )
  	  outf.write(template_str % display_entry)
      else:
        raise w2lcore.ErrorExitClass(
          ls,
          """vCard template file for object class
          <strong>%s</strong> not found.""" % (oc)
        )

    else:
      raise w2lcore.ErrorExitClass(
        ls,"""No template file defined for object
        class(es) <strong>%s</strong>.""" % (', '.join(objectclasses))
      )
