"""
w2lreferral.py: chase LDAP referrals

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import sys,string,ldap,ldapbase,w2lcore,w2lgui,web2ldapcnf

def w2l_ChaseReferral(sid,outf,command,form,ls,e):

  w2lgui.PrintHeader(outf,'Referral received',form.accept_charset)
  w2lgui.MainMenuTable(sid,outf,form,ls)

  # Pull out referral LDAP URL
  try:
    info, ldap_url_info = map(string.strip,string.split(e.args[0].get('info',''),'\n',1))
  except ValueError:
    outf.write("""
    <div id="ErrorMessageDiv">
      <h1>Error</h1>
      <p>Error extracting referral LDAP URL.</p>
    </div>
    """)
    return
  else:
    ldap_url = ldap_url_info[string.find(ldap_url_info,'ldap:'):]
    # Parse the referral LDAP URL
    referral_host,referral_dn,search_attr,search_scope,search_filterstr,ext = ldapbase.parse_ldap_url(ldap_url)
    # DN which triggered the referral
    matched_dn = e.args[0].get('matched','')
    matched_dn_list = ldapbase.explode_dn(matched_dn)
    referral_dn_list = ldapbase.explode_dn(referral_dn)
    # determine new DN
    ls.dn_list=ldapbase.explode_dn(ls.dn)
    new_dn = string.join(
      ls.dn_list[0:len(ls.dn_list)-len(matched_dn_list)]+referral_dn_list,
      ','
    )

  outf.write("""<div id=InputDiv>
<h1>Referral received</h1>
<p>
  Referral URL: %s
</p>
<p>
  Referral DN %s matches %s =&gt; %s
</p>
"""  % (
      ldap_url,
      w2lgui.DisplayDN(sid,form,ls,referral_dn),
      w2lgui.DisplayDN(sid,form,ls,matched_dn),
      w2lgui.DisplayDN(sid,form,ls,new_dn),
    )
  )
  if command=='ldapurl':
    outf.write("""<h2><a href="%s/ldapurl?%s">Chase Referral</a></h2>
""" % (
        form.script_name,
        ldapbase.create_ldap_url(referral_host,new_dn,search_attr,search_scope,search_filterstr,ext,urlencode=1)
      )
    )
  else:
    outf.write("""
<form
  id=CommandForm
  action="%s/%s"
  method="POST"
  enctype="application/x-www-form-urlencoded"
  accept-charset="%s"
>
  %s
  <input type="hidden" name="sid" value="%s">
  <input type="hidden" name="dn" value="%s">
  <input type="hidden" name="host" value="%s">
  <p>host:port %s (%s)</p>
  <input type="submit" value="Chase Referral">
"""  % (
        form.script_name,
        command,
        form.accept_charset,
        w2lgui.BindInput % (''),
        sid,
        w2lcore.utf2display(form.accept_charset,new_dn),
        referral_host,
        referral_host,
        w2lcore.iso2display(
          form.accept_charset,
          web2ldapcnf.hosts.ldap_def.get(referral_host,{}).get('description','')
        ),
      )
    )
    form.hiddeninputparams(outf,['sid','host','dn','who','cred'])
    outf.write('</form></div>\n')

  w2lgui.PrintFooter(outf)
