"""
w2lsearchform.py: different search forms

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>
Distributed under GNU Public License (GPL)
"""

import string,cgiforms,w2lcore,web2ldapcnf,w2lgui,w2lcnf


searchform_mode_text = {
  'adv':'Advanced',
  'base':'Basic',
  'exp':'Expert',
}

def SearchForm_begin(sid,outf,command,form,ls,searchform_mode,Msg=''):
  outf.write("""
    <div id=InputDiv><h1>%s Search Form</h1>
      <form
        action="%s/search"
        method="%s"
        enctype="application/x-www-form-urlencoded"
        accept-charset="%s"
      >
      <input type="hidden" name="sid" value="%s">
      %s
      """ % (
      searchform_mode_text[searchform_mode],
      form.script_name,
      web2ldapcnf.script_method,
      form.accept_charset,
      sid,
      Msg,
    )
  )


def SearchForm_exp(sid,outf,command,form,ls,search_filterstr='()'):
  """Output expert search form"""
  # expert search form for using LDAP filters
  if 'search_filterstr' in form.inputkeys:
    search_filterstr = form.field['search_filterstr'][0].content
  outf.write("""
    LDAP filter string<BR>
    <input name="search_filterstr" maxlength="120" size="60" value="%s">
    """ % (
      search_filterstr.encode(form.accept_charset),
    )
  )


def SearchForm_base(sid,outf,command,form,ls):
  """Output basic search form"""
  searchform_template_filename = w2lgui.GetVariantFilename(
    w2lcnf.GetParam(ls,'searchform_template',''),
    form.accept_language
  )
  if searchform_template_filename:
    try:
      template_str = open(searchform_template_filename,'r').read()
    except IOError:
      outf.write('<div id=MessageDiv><p>I/O error during reading template file.</p></div>')
    else:
      outf.write(template_str)
  else:
    outf.write('<div id=MessageDiv><p>Template file not found.</p></div>')


def SearchForm_adv(sid,outf,command,form,ls):
  """Output basic search form"""
  # advanced search form with select lists
  outf.write('<div id=InputDiv><h1>Advanced Search Form</h1>')

  if ('search_params' in form.inputkeys) and \
     form.field['search_params'][0].content:
    # Gewuenschte Anzahl der Suchparameter wurde angegeben
    rows = string.atoi(form.field['search_params'][0].content)
    if rows<1:                rows=1
    if rows>web2ldapcnf.max_searchparams: rows=web2ldapcnf.max_searchparams
  else:
    rows = 1

  # More and Fewer-Buttons
  outf.write('<table><tr>')
  if rows<web2ldapcnf.max_searchparams:
    outf.write(
      '<td>%s</td>' % w2lgui.CommandButton(
	form,'searchform','More',sid,
        hidden_fields=[
          ('searchform_mode','adv'),
          ('search_params',str(rows+1)),
        ],
      )
    )
  if rows>1:
    outf.write('<td>%s</td>' % w2lgui.CommandButton(
        form,'searchform','Fewer',sid,
        hidden_fields=[
          ('searchform_mode','adv'),
          ('search_params',str(rows-1)),
        ],
      )
    )

  outf.write('<td>search parameters</td></tr></table>')

  search_mode_select = cgiforms.formSelectClass(
    'search_mode',
    u'Search mode',
    [(ur'(&%s)',u'all'),(ur'(|%s)',u'any')]
  )
  search_mode_select.setcharset(form.accept_charset)
  search_attrs = w2lcnf.GetParam(ls,'search_attrs',['cn','mail'])

  # Eigentliches Suchformular ausgeben
  outf.write("""
    <form 
      action="%s/search"
      method="%s"
      enctype="application/x-www-form-urlencoded"
      accept-charset="%s"
    >
    <input type="hidden" name="sid" value="%s">
    Match %s of the following.<br>
    """ % (
      form.script_name,
      web2ldapcnf.script_method,
      form.accept_charset,
      sid,
      search_mode_select.inputfield()
    )
  )

  search_attr_list = []
  for attr in search_attrs:
    search_attr_list.append((attr,w2lcore.ldap_knownattr.get(attr,attr)))
  search_attr_select = cgiforms.formSelectClass('search_attr',u'Search attribute type',search_attr_list,multiple=web2ldapcnf.max_searchparams)
  search_attr_select.setcharset(form.accept_charset)
  search_option_select = cgiforms.formSelectClass('search_option','Search option',w2lcore.search_options,multiple=web2ldapcnf.max_searchparams)
  search_option_select.setcharset(form.accept_charset)
  # Output a row of the search form
  for i in range(rows):
    search_attr_select.setdefault(search_attrs[i % len(search_attrs)])
    outf.write('%s\n%s\n' % (search_attr_select.inputfield(),search_option_select.inputfield()))
    # Output of search string input field
    outf.write('<input name="search_string" maxlength="60" size="30" value=""><br>\n')


def w2l_SearchForm(sid,outf,command,form,ls,searchform_mode='base',Msg='',search_filterstr='()'):
  """Output a search form"""

  if 'searchform_mode' in form.inputkeys:
    searchform_mode = form.field['searchform_mode'][0].content

  w2lgui.PrintHeader(
    outf,
    '%s Search Form' % searchform_mode_text[searchform_mode],
    form.accept_charset
  )
  w2lgui.MainMenuTable(sid,outf,form,ls)

  if searchform_mode == 'base':
    # base search form with fixed input fields
    SearchForm_begin(sid,outf,command,form,ls,searchform_mode,Msg)
    SearchForm_base(sid,outf,command,form,ls)
  elif searchform_mode == 'exp':
    # expert search form with single filter input field
    SearchForm_begin(sid,outf,command,form,ls,searchform_mode,Msg)
    SearchForm_exp(sid,outf,command,form,ls,search_filterstr)
  elif searchform_mode == 'adv':
    # base search form with fixed input fields
    SearchForm_adv(sid,outf,command,form,ls)

  search_root_field = w2lgui.SearchRootField(form,ls)
  if 'search_root' in form.inputkeys:
    search_root_field.setdefault(form.field['search_root'][0].content)

  if 'search_scope' in form.inputkeys:
    form.field['search_scope'][0].setdefault(form.field['search_scope'][0].content)

  search_resnumber_field = cgiforms.formSelectClass('search_resnumber',u'Number of results to display',[(u'0',u'unlimited'),(u'10',u'10'),(u'20',u'20'),(u'50',u'50'),(u'100',u'100'),(u'200',u'200')],u'10')

  # Output of form part for all kind of search forms
  outf.write("""
<p>
 Search root
 %s
 Scope %s
 Results per page %s
</p>
 <input type="submit" value="Search">
 <input type="reset" value="Reset">
 </form>
</div>
""" % (
    search_root_field.inputfield(),
    form.field['search_scope'][0].inputfield(),
    search_resnumber_field.inputfield(),
  ))

  w2lgui.PrintFooter(outf)

